#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <rsbac/types.h>
#include <rsbac/aci_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>

int main(int argc, char ** argv)
{
  int attr_list[RSBAC_PROCESS_NR_ATTRIBUTES] = RSBAC_PROCESS_ATTR_LIST;
  int res = 0;
  char tmp1[80],tmp2[120];
  int j;
  int position, catval;
  rsbac_mac_category_vector_t k;
  union rsbac_attribute_value_t value;
  union rsbac_target_id_t tid;
  enum rsbac_attribute_t attr;
  
  switch(argc)
    {
      case 4:
        attr = get_attribute_nr(argv[2]);
        if(attr == A_none)
          {
            fprintf(stderr, "%s: Invalid Attribute %s!\n", argv[0], argv[2]);
            exit(1);
          }
        switch(attr)
          {
            case A_owner:
              value.owner = strtol(argv[3],0,0);
              break;
            case A_mac_categories:
              if(strlen(argv[4]) != RSBAC_MAC_NR_CATS)
                {
                  fprintf(stderr, "%s: Invalid attribute value, length must be %i\n", argv[0],
                          RSBAC_MAC_NR_CATS);
                  exit(1);
                }
              for(j=0;j<strlen(argv[4]);j++)
                {
                  if(   argv[4][j] != '0'
                     && argv[4][j] != '1')
                    {
                      fprintf(stderr, "%s: Invalid attribute value char, must be 0 or 1\n", argv[0]);
                      exit(1);
                    }
                }
              strtou64mac(argv[4], &value.mac_categories);
              break;

            default: value.dummy = strtol(argv[3],0,0);
          }
        tid.process = strtol(argv[1],0,0);
        res = rsbac_set_attr(T_PROCESS, &tid, attr, &value);
        error_exit(res);
        exit(0);

      case 5:
        attr = get_attribute_nr(argv[2]);
        switch(attr)
          {
            case A_mac_categories:
            case A_mac_curr_categories:
            case A_max_read_categories:
            case A_min_write_categories:
              break;
            default:
              fprintf(stderr, "%s: Invalid Attribute %s!\n", argv[0], argv[2]);
              exit(1);
          }
        tid.process = strtol(argv[1],0,0);
        position = strtol(argv[3],0,10);
        if(position > RSBAC_MAC_MAX_CAT)
          {
            fprintf(stderr, "Invalid position counter %s\n", argv[3]);
            exit(1);
          }
        catval = strtol(argv[4],0,0);
        if(catval > 1)
          {
            fprintf(stderr, "Invalid value %s\n", argv[4]);
            exit(1);
          }
        res = rsbac_get_attr(T_PROCESS, &tid, attr, &value, FALSE);
        error_exit(res);
        k = ((__u64) 1) << position;
        if(catval)
          value.mac_categories |= k;
        else
          value.mac_categories &= ~k;
        res = rsbac_set_attr(T_PROCESS, &tid, attr, &value);
        error_exit(res);
        exit(0);

      default:
        printf("%s (RSBAC %s)\n***\n", argv[0], VERSION);
        printf("Use: %s process-id attribute value\n", argv[0]);  
        printf("- attribute (string) and value (integer) = see following list:\n");
        for (j=0;j<RSBAC_PROCESS_NR_ATTRIBUTES;j++)
          {
            get_attribute_name(tmp1,attr_list[j]);
            get_attribute_param(tmp2,attr_list[j]);
            printf("%s \t%s\n",tmp1,tmp2);
          }
    }
  exit(1);
}
